@@ -5,7 +5,13 @@ use warnings;
 # TODO: Write this.
 package WWW::Shopify::Liquid::Filter::ProductImgUrl;
 use base 'WWW::Shopify::Liquid::Filter';
+sub min_arguments { return 1; }
 sub max_arguments { return 1; }
-sub operate { return '<a href="/customers/login">' . $_[2] .  '</a>'; }
+sub operate { 
+	my ($self, $hash, $operand, @arguments) = @_;
+	my $size = $arguments[0];
+	$operand =~ s/(\.(jpg|png|jpeg|gif))/_$size$1/i;
+	return $operand;
+}
 
 1;
\ No newline at end of file
