@@ -5,7 +5,7 @@ use warnings;
 package WWW::Shopify::Liquid::Operator::Contains;
 use base 'WWW::Shopify::Liquid::Operator';
 sub symbol { return 'contains'; }
-sub priority { return 2; }
-sub operate { return ($_[2] cmp $_[3]) != 1; }
+sub priority { return 11; }
+sub operate { return index($_[3], $_[4]) != -1; }
 
 1;
\ No newline at end of file
