@@ -50,7 +50,7 @@ sub register_operator {
 	else {
 		push(@$ooo, [$_[1]]);
 	}
-	$_[0]->{order_of_operations} = [sort { $a->[0]->priority <=> $b->[0]->priority } @$ooo];
+	$_[0]->{order_of_operations} = [sort { $b->[0]->priority <=> $a->[0]->priority } @$ooo];
 }
 sub register_filter {
 	$_[0]->filters->{$_[1]->name} = $_[1];
@@ -62,61 +62,91 @@ sub parse_filter_tokens {
 	my ($self, $initial, @tokens) = @_;
 	my $filter = shift(@tokens);
 	my $filter_name = $filter->{core}->[0]->{core};
-	die new WWW::Shopify::Liquid::Exception::Parser::UnknownFilter($filter) unless $self->{filters}->{$filter_name};
+	# TODO God, this is stupid, but temporary patch.
+	my $filter_package;
+	if ($filter_name =~ m/::/) {
+		$filter_package = $filter_name;
+		eval { $filter_package->name };
+		die new WWW::Shopify::Liquid::Exception::Parser::UnknownFilter($filter) if $@;
+	} else {
+		die new WWW::Shopify::Liquid::Exception::Parser::UnknownFilter($filter) unless $self->{filters}->{$filter_name};
+		$filter_package = $self->{filters}->{$filter_name};
+	}
 	die new WWW::Shopify::Liquid::Exception::Parser::Arguments($filter, "In order to have arguments, filter must be followed by a colon.") if int(@tokens) > 0 && $tokens[0]->{core} ne ":";
 	
 	my @arguments = ();
+	# Get rid of our colon.
 	if (shift(@tokens)) {
 		my $i = 0;
-		@arguments = map { shift(@{$_}) if $_->[0]->{core} eq "|"; $self->parse_argument_tokens(@{$_}) } part { $i++ if $_->isa('WWW::Shopify::Liquid::Token::Separator') && $_->{core} eq ","; $i; } @tokens;
+		@arguments = map { $self->parse_argument_tokens(grep { !$_->isa('WWW::Shopify::Liquid::Token::Separator') } @{$_}) } part { $i++ if $_->isa('WWW::Shopify::Liquid::Token::Separator') && $_->{core} eq ","; $i; } @tokens;
 	}
-	$filter = $self->{filters}->{$filter_name}->new($initial, @arguments);
+	$filter = $filter_package->new($initial, @arguments);
 	$filter->verify;
 	return $filter;
 }
+use List::MoreUtils qw(part);
 
 # Similar, but doesn't deal with tags; deals solely with order of operations.
 sub parse_argument_tokens {
-	my ($self, @tokens) = @_;
+	my ($self, @argument_tokens) = @_;
 	# First, pull together filters. These are the highest priority operators, after parentheses. They also have their own weird syntax.
 	my $top = undef;
 	
-	# Use the order of operations to create a binary tree structure.
-	foreach my $operators ($self->order_of_operations) {
-		# If we have pipes, we deal with those, and prase their lower level arguments.
-		foreach my $operator (@$operators) {
-			my %ops = map { $_ => 1 } $operator->symbol;
-			if ($operator eq 'WWW::Shopify::Liquid::Operator::Pipe') {
-				if ((my $idx = firstidx { $_->isa('WWW::Shopify::Liquid::Token::Operator') && $_->{core} eq "|" } @tokens) != -1) {
-					die new WWW::Shopify::Liquid::Exception::Parser($tokens[0]) if $idx == 0;
-					my $i = 0;
-					# Part should consist of the first token before a pipe, and then split on all pipes after this.,
-					my @parts = map { shift(@{$_}) if $_->[0]->{core} eq "|"; $_ } part { $i++ if $_->isa('WWW::Shopify::Liquid::Token::Operator') && $_->{core} eq "|"; $i; } splice(@tokens, $idx-1);
-					my $next = undef;
-					$top = $self->parse_filter_tokens($self->parse_argument_tokens(@{shift(@parts)}), @{shift(@parts)});
-					while (my $part = shift(@parts)) {
-						$top = $self->parse_filter_tokens($top, @$part);
+	# Don't partition if we have any pipes. Pipes and multiple arguments don't play well together.
+	my @partitions;
+	my $has_pipe = 0;
+	if (int(grep { $_->isa('WWW::Shopify::Liquid::Token::Operator') && $_->{core} eq "|" } @argument_tokens) == 0)  {
+		my $i = 0;
+		$has_pipe = 1;
+		@partitions = part { $i++ if $_->isa('WWW::Shopify::Liquid::Token::Separator'); $i; } @argument_tokens;
+	} else {
+		@partitions = (\@argument_tokens);
+	}
+	
+	my @tops;
+	foreach my $partition (@partitions) {
+		my @tokens = @$partition;
+		#@tokens = (grep { !$_->isa('WWW::Shopify::Liquid::Token::Separator') } @tokens) if !$has_pipe;
+		
+		# Use the order of operations to create a binary tree structure.
+		foreach my $operators ($self->order_of_operations) {
+			# If we have pipes, we deal with those, and parse their lower level arguments.
+			foreach my $operator (@$operators) {
+				my %ops = map { $_ => 1 } $operator->symbol;
+				if ($operator eq 'WWW::Shopify::Liquid::Operator::Pipe') {
+					if ((my $idx = firstidx { $_->isa('WWW::Shopify::Liquid::Token::Operator') && $_->{core} eq "|" } @tokens) != -1) {
+						die new WWW::Shopify::Liquid::Exception::Parser($tokens[0]) if $idx == 0;
+						my $i = 0;
+						# Part should consist of the first token before a pipe, and then split on all pipes after this.,
+						my @parts = map { shift(@{$_}) if $_->[0]->{core} eq "|"; $_ } part { $i++ if $_->isa('WWW::Shopify::Liquid::Token::Operator') && $_->{core} eq "|"; $i; } splice(@tokens, $idx-1);
+						my $next = undef;
+						$top = $self->parse_filter_tokens($self->parse_argument_tokens(@{shift(@parts)}), @{shift(@parts)});
+						while (my $part = shift(@parts)) {
+							$top = $self->parse_filter_tokens($top, @$part);
+						}
+						push(@tokens, $top);
 					}
-					push(@tokens, $top);
 				}
-			}
-			else {
-				while ((my $idx = firstidx { $_->isa('WWW::Shopify::Liquid::Token::Operator') && exists $ops{$_->{core}} } @tokens) != -1) {
-					my ($op1, $op, $op2) = @tokens[$idx-1..$idx+1];
-					die new WWW::Shopify::Liquid::Exception::Parser::Operands($tokens[0]) unless
-						$idx > 0 && $idx < $#tokens && 
-						($op1->isa('WWW::Shopify::Liquid::Operator') || $op1->isa('WWW::Shopify::Liquid::Token::Operand') || $op1->isa('WWW::Shopify::Liquid::Filter')) &&
-						($op2->isa('WWW::Shopify::Liquid::Operator') || $op2->isa('WWW::Shopify::Liquid::Token::Operand') || $op2->isa('WWW::Shopify::Liquid::Filter'));
-					$op1 = $self->parse_argument_tokens($op1->members) if $op1->isa('WWW::Shopify::Liquid::Token::Grouping');
-					$op2 = $self->parse_argument_tokens($op2->members) if $op2->isa('WWW::Shopify::Liquid::Token::Grouping');
-					splice(@tokens, $idx-1, 3, $self->operators->{$op->{core}}->new($op->{core}, $op1, $op2));
+				else {
+					while ((my $idx = firstidx { $_->isa('WWW::Shopify::Liquid::Token::Operator') && exists $ops{$_->{core}} } @tokens) != -1) {
+						my ($op1, $op, $op2) = @tokens[$idx-1..$idx+1];
+						# The one exception would be if we have a - operator, and nothing before, this is unary negative operator, i.e. 0 - number.
+						die new WWW::Shopify::Liquid::Exception::Parser::Operands($tokens[0]) unless
+							$idx > 0 && $idx < $#tokens && 
+							($op1->isa('WWW::Shopify::Liquid::Operator') || $op1->isa('WWW::Shopify::Liquid::Token::Operand') || $op1->isa('WWW::Shopify::Liquid::Filter')) &&
+							($op2->isa('WWW::Shopify::Liquid::Operator') || $op2->isa('WWW::Shopify::Liquid::Token::Operand') || $op2->isa('WWW::Shopify::Liquid::Filter'));
+						($op1) = $self->parse_argument_tokens($op1->members) if $op1->isa('WWW::Shopify::Liquid::Token::Grouping');
+						($op2) = $self->parse_argument_tokens($op2->members) if $op2->isa('WWW::Shopify::Liquid::Token::Grouping');
+						splice(@tokens, $idx-1, 3, $self->operators->{$op->{core}}->new($op->{core}, $op1, $op2));
+					}
 				}
 			}
 		}
+		
+		die new WWW::Shopify::Liquid::Exception::Parser::Operands($tokens[0]) unless int(@tokens) == 1 || int(@tokens) == 0;
+		push(@tops, $tokens[0]);
 	}
-	die new WWW::Shopify::Liquid::Exception::Parser::Operands($tokens[0]) unless int(@tokens) == 1 || int(@tokens) == 0;
-	($top) = @tokens;
-	return $top;
+	return @tops;
 }
 
 sub parse_tokens {
@@ -135,12 +165,13 @@ sub parse_tokens {
 				my @contents = ();
 				my %allowed_internal_tags = map { $_ => 1 } $self->enclosing_tags->{$token->tag}->inner_tags;
 				my $level = 1;
+				my $closed = undef;
 				for (0..$#tokens) {
 					if ($tokens[$_]->isa('WWW::Shopify::Liquid::Token::Tag')) {
 						if ($tokens[$_]->tag eq $token->tag) {
 							$level++;
 						} elsif (exists $allowed_internal_tags{$tokens[$_]->tag} && $level == 1) {
-							$tokens[$_]->{arguments} = $self->parse_argument_tokens(@{$tokens[$_]->{arguments}});
+							$tokens[$_]->{arguments} = [$self->parse_argument_tokens(@{$tokens[$_]->{arguments}})];
 							push(@internal, $_);
 						} elsif ($tokens[$_]->tag eq "end" . $token->tag && --$level == 0) {
 							my $last_int = 0;
@@ -160,16 +191,17 @@ sub parse_tokens {
 							}
 							# Remove the endtag.
 							shift(@tokens);
+							$closed = 1;
 							last;
 						}
 					}
-					die new WWW::Shopify::Liquid::Exception::Parser::NoClose($token) if $_ == $#tokens && $level > 0;
 				}
-				$tag = $self->enclosing_tags->{$token->tag}->new($line, $token->tag, $self->parse_argument_tokens(@{$token->{arguments}}), \@contents);
+				die new WWW::Shopify::Liquid::Exception::Parser::NoClose($token) unless $closed;
+				$tag = $self->enclosing_tags->{$token->tag}->new($line, $token->tag, [$self->parse_argument_tokens(@{$token->{arguments}})], \@contents);
 				$tag->verify;
 			}
 			elsif ($self->free_tags->{$token->tag}) {
-				$tag = $self->free_tags->{$token->tag}->new($line, $token->tag, $self->parse_argument_tokens(@{$token->{arguments}}));
+				$tag = $self->free_tags->{$token->tag}->new($line, $token->tag, [$self->parse_argument_tokens(@{$token->{arguments}})]);
 				$tag->verify;
 			}
 			else {
@@ -180,7 +212,7 @@ sub parse_tokens {
 			push(@tags, $tag);
 		}
 		elsif ($token->isa('WWW::Shopify::Liquid::Token::Output')) {
-			push(@tags, WWW::Shopify::Liquid::Tag::Output->new($line, $self->parse_argument_tokens(@{$token->{core}})));
+			push(@tags, WWW::Shopify::Liquid::Tag::Output->new($line, [$self->parse_argument_tokens(@{$token->{core}})]));
 		}
 		else {
 			push(@tags, $token);
@@ -212,8 +244,8 @@ sub parse_tokens {
 sub unparse_argument_tokens {
 	my ($self, $ast) = @_;
 	return $ast if $ast->isa('WWW::Shopify::Liquid::Token');
-	my @optokens = ($self->unparse_argument_tokens($ast->{operands}->[0]), WWW::Shopify::Liquid::Token::Operator->new($ast->{core}), $self->unparse_argument_tokens($ast->{operands}->[1]));
-	return WWW::Shopify::Liquid::Token::Grouping->new(@optokens) if $ast->requires_grouping;
+	my @optokens = ($self->unparse_argument_tokens($ast->{operands}->[0]), WWW::Shopify::Liquid::Token::Operator->new([0,0,0], $ast->{core}), $self->unparse_argument_tokens($ast->{operands}->[1]));
+	return WWW::Shopify::Liquid::Token::Grouping->new([0,0,0], @optokens) if $ast->requires_grouping;
 	return @optokens;
 }
 
@@ -221,21 +253,21 @@ sub unparse_tokens {
 	my ($self, $ast) = @_;
 	return $ast if $ast->isa('WWW::Shopify::Liquid::Token');
 	if ($ast->isa('WWW::Shopify::Liquid::Tag')) {
-		my @arguments = $ast->{arguments} ? $self->unparse_argument_tokens($ast->{arguments}) : ();
+		my @arguments = $ast->{arguments} ? $self->unparse_argument_tokens(@{$ast->{arguments}}) : ();
 		if ($ast->isa('WWW::Shopify::Liquid::Tag::Enclosing')) {
 			if ($ast->isa('WWW::Shopify::Liquid::Tag::If')) {
-				return (WWW::Shopify::Liquid::Token::Tag->new('if', \@arguments), $self->unparse_tokens($ast->{true_path}), WWW::Shopify::Liquid::Token::Tag->new('endif')) if !$ast->{false_path};
-				return (WWW::Shopify::Liquid::Token::Tag->new('if', \@arguments), $self->unparse_tokens($ast->{true_path}), WWW::Shopify::Liquid::Token::Tag->new('else'), $self->unparse_tokens($ast->{false_path}), WWW::Shopify::Liquid::Token::Tag->new('endif'));
+				return (WWW::Shopify::Liquid::Token::Tag->new([0,0,0], 'if', \@arguments), $self->unparse_tokens($ast->{true_path}), WWW::Shopify::Liquid::Token::Tag->new([0,0,0], 'endif')) if !$ast->{false_path};
+				return (WWW::Shopify::Liquid::Token::Tag->new([0,0,0], 'if', \@arguments), $self->unparse_tokens($ast->{true_path}), WWW::Shopify::Liquid::Token::Tag->new([0,0,0], 'else'), $self->unparse_tokens($ast->{false_path}), WWW::Shopify::Liquid::Token::Tag->new([0,0,0], 'endif'));
 			}
 			else {
-				return (WWW::Shopify::Liquid::Token::Tag->new($ast->{core}, \@arguments), $self->unparse_tokens($ast->{contents}), WWW::Shopify::Liquid::Token::Tag->new('end' . $ast->{core}));
+				return (WWW::Shopify::Liquid::Token::Tag->new([0,0,0], $ast->{core}, \@arguments), $self->unparse_tokens($ast->{contents}), WWW::Shopify::Liquid::Token::Tag->new([0,0,0], 'end' . $ast->{core}));
 			}
 		}
 		elsif ($ast->isa('WWW::Shopify::Liquid::Tag::Output')) {
-			return (WWW::Shopify::Liquid::Token::Output->new([$self->unparse_argument_tokens($ast->{arguments})]));
+			return (WWW::Shopify::Liquid::Token::Output->new([0,0,0], [$self->unparse_argument_tokens(@{$ast->{arguments}})]));
 		}
 		else  {
-			return (WWW::Shopify::Liquid::Token::Tag->new($ast->{core}, \@arguments));
+			return (WWW::Shopify::Liquid::Token::Tag->new([0,0,0], $ast->{core}, \@arguments));
 		}
 		return $ast;
 	}
