@@ -18,9 +18,14 @@ sub is_enclosing { return 0; }
 sub min_arguments { return 0; }
 sub max_arguments { return undef; }
 
+sub tokens { return ($_[0], map { $_->tokens } grep { defined $_ } (@{$_[0]->{arguments}}, $_[0]->{contents})) }
+
 package WWW::Shopify::Liquid::Tag::Output;
 use base 'WWW::Shopify::Liquid::Tag::Free';
 sub abstract { my $package = ref($_[0]) ? ref($_[0]) : $_[0]; return ($package eq __PACKAGE__); }
+
+sub max_arguments { return 1; }
+
 sub new { 
 	my ($package, $line, $arguments) = @_;
 	my $self = { arguments => $arguments, line => $line };
@@ -28,7 +33,8 @@ sub new {
 }
 sub process {
 	my ($self, $hash, $action) = @_;
-	return $self->{arguments}->$action($hash);
+	return '' unless int(@{$self->{arguments}}) > 0;
+	return $self->{arguments}->[0]->$action($hash);
 }
 
 1;
